unit OOView;

{$mode delphi}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, ActnList,
  ExtCtrls, StdCtrls;

type
  TStatusBarWidget = class
  private
    fOrigin: TStatusBar;
  public
    constructor Create(aOrigin: TStatusBar); reintroduce;
    procedure ShowVersion;
    procedure ShowStatus(const aValue: string);
    procedure ClearStatus;
  end;

  TCar = packed record
    company: string;
    model: string;
    km: Integer;
  end;

  TCarListViewWidget = class
  private
    fOrigin: TListView;
  public
    constructor Create(aOrigin: TListView); reintroduce;
    procedure Insert(const aCar: TCar);
  end;

type
  TOOForm = class; // forward declaration

  TCrudModeWidgets = class
  private
    fOrigin: TOOForm;
  public
    constructor Create(aOrigin: TOOForm); reintroduce;
    procedure Inserting;
    procedure Viewing;
  end;

  TOOForm = class(TForm)
    StatusBar: TStatusBar;
    ToolBar1: TToolBar;
    ImageList: TImageList;
    ToolButton2: TToolButton;
    ActionList: TActionList;
    SaveAction: TAction;
    EditPanel: TPanel;
    ListView: TListView;
    CompanyEdit: TEdit;
    ModelEdit: TEdit;
    KmEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ToolButton3: TToolButton;
    NewAction: TAction;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    CancelAction: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure NewActionExecute(Sender: TObject);
    procedure SaveActionExecute(Sender: TObject);
    procedure CancelActionExecute(Sender: TObject);
  private
    fStatusBar: TStatusBarWidget;
    fCrudMode: TCrudModeWidgets;
    fCars: TCarListViewWidget;
  end;

var
  OOForm: TOOForm;

implementation

{$R *.lfm}

{ TStatusBarWidget }

constructor TStatusBarWidget.Create(aOrigin: TStatusBar);
begin
  inherited Create;
  fOrigin := aOrigin;
end;

procedure TStatusBarWidget.ShowVersion;
begin
  fOrigin.Panels[0].Text := '1.0'; // get version using some object
end;

procedure TStatusBarWidget.ShowStatus(const aValue: string);
begin
  fOrigin.Panels[1].Text := aValue;
  fOrigin.Repaint;
end;

procedure TStatusBarWidget.ClearStatus;
begin
  ShowStatus('');
end;

{ TCrudModeWidgets }

constructor TCrudModeWidgets.Create(aOrigin: TOOForm);
begin
  fOrigin := aOrigin;
end;

procedure TCrudModeWidgets.Inserting;
begin
  with fOrigin do
  begin
    CompanyEdit.Clear;
    ModelEdit.Clear;
    KmEdit.Clear;
    EditPanel.Visible := True;
    NewAction.Enabled := False;
    SaveAction.Enabled := True;
    CancelAction.Enabled := True;
  end;
end;

procedure TCrudModeWidgets.Viewing;
begin
  with fOrigin do
  begin
    EditPanel.Visible := False;
    NewAction.Enabled := True;
    SaveAction.Enabled := False;
    CancelAction.Enabled := False;
  end;
end;

{ TCarListViewWidget }

constructor TCarListViewWidget.Create(aOrigin: TListView);
begin
  inherited Create;
  fOrigin := aOrigin;
end;

procedure TCarListViewWidget.Insert(const aCar: TCar);
var
  item: TListItem;
begin
  item := fOrigin.Items.Add;
  item.SubItems.Add(aCar.company);
  item.SubItems.Add(aCar.model);
  item.SubItems.Add(aCar.km.ToString);
end;

{ TOOForm }

procedure TOOForm.FormCreate(Sender: TObject);
begin
  fStatusBar := TStatusBarWidget.Create(StatusBar);
  fStatusBar.ShowVersion;
  fCrudMode := TCrudModeWidgets.Create(self);
  fCars := TCarListViewWidget.Create(ListView);
end;

procedure TOOForm.FormDestroy(Sender: TObject);
begin
  fStatusBar.Free;
  fCrudMode.Free;
  fCars.Free;
end;

procedure TOOForm.NewActionExecute(Sender: TObject);
begin
  fCrudMode.Inserting;
end;

procedure TOOForm.SaveActionExecute(Sender: TObject);
var
  car: TCar;
begin
  if not TryStrToInt(KmEdit.Text, car.km) then
  begin
    MessageDlg(Caption, 'Invalid Km/h', mtWarning, [mbOK], '');
    exit;
  end;
  car.company := CompanyEdit.Text;
  car.model := ModelEdit.Text;
  fCars.Insert(car);

  fStatusBar.ShowStatus('Saving...');
  Sleep(1500); // database persistence simulation
  fStatusBar.ClearStatus;
  fCrudMode.Viewing;

  MessageDlg(Caption, 'Saved', mtInformation, [mbOK], '');
end;

procedure TOOForm.CancelActionExecute(Sender: TObject);
begin
  if MessageDlg(Caption, 'Cancel?', mtConfirmation, mbYesNo, 0, mbNo) = mrNo then
    exit;
  fCrudMode.Viewing;
end;

end.
